declare type Data = Record<string, unknown>;
interface PackageJson extends Data {
    dependencies?: Data;
    devDependencies?: Data;
    version: string;
}
interface Options {
    targetDir: string;
    pkg: PackageJson;
}
declare type GetSetPath = string | string[];
declare type ContentsUpdater = (fileContents: string) => (Promise<string> | string);
declare type JSONContentsUpdater = (fileData: Data, setFn: (objPath: GetSetPath, value: unknown) => void) => (Promise<void> | void);
declare class Migration {
    private readonly _targetDir;
    private readonly _pkg;
    constructor(opts: Options);
    private _savePkg;
    setPath(objPath: GetSetPath, value: unknown): Promise<void>;
    setScript(scriptName: string, value: string): Promise<void>;
    upgradeScript(scriptName: string, oldValue: string, newValue: string): Promise<void>;
    addDependency(name: string, version: string): Promise<void>;
    addDevDependency(name: string, version: string): Promise<void>;
    upgradeDependency(name: string, version: string, defaultType?: string): Promise<void>;
    findDependency(name: string): "dependencies" | "devDependencies" | undefined;
    remove(dirName: string): Promise<void>;
    copy(sourceName: string, targetName?: string | null): Promise<void>;
    rename(source: string, target: string): Promise<void>;
    updateContents(file: string, updater: ContentsUpdater): Promise<void>;
    updateContentsJSON(file: string, updater: JSONContentsUpdater): Promise<void>;
    assertPath(objPath: GetSetPath, value: unknown, error: Error): void;
    assertScript(scriptName: string, value: string, error: Error): void;
    assertNoDependency(name: string, error: Error): void;
    assertNoDevDependency(name: string, error: Error): void;
    assertNoAnyDependency(name: string, error: Error): void;
    yarn(): Promise<void>;
}
export { Migration };
export type { PackageJson };
//# sourceMappingURL=Migration.d.ts.map